<?php

namespace Evoluted\Deployer\VhostApi;

use function Deployer\has;
use function Deployer\get;
use function Deployer\writeln;

/**
 * Notify the Vhost API about a (possibly new) hostname
 *
 * @param string $hostname
 * @param string $applicationType E.g. "laravel", "cake"
 * @param string $phpVersion E.g. "7.1"
 * @param string $serverType Not currently used
 * @return bool Whether the request to the vhost API was successful
 */
function notifyVhostApiManager($hostname, $applicationType, $phpVersion, $serverType) {

    // If it is UAT
    if (has('vhost_api_host')) {
        $vhostApiManagerHost = get('vhost_api_host');
    } elseif (strtolower(getenv('CI_BUILD_REF_SLUG')) == 'uat' || strtolower(getenv('IS_UAT') == 'yes')) {
        $vhostApiManagerHost = 'https://vhost-api.uat.evoluted.net';
    } else {
        $vhostApiManagerHost = 'http://vhost-api.cd.evoluted.net';
    }

    if (has('application/public_path')) {
        $applicationPublicPath = get('application/public_path');
    } else {
        $applicationPublicPath = getApplicationPublicPath($applicationType);
    }

    $bodyArray = [
        'template' => 'nginx',
        'server_name' => $hostname,
        'root' => get('release_path') . DIRECTORY_SEPARATOR . $applicationPublicPath,
        'server_type' => 'php-fpm',
        'php_version' => $phpVersion
    ];

    // Merge in any custom params added by the applications deployer config.
    if (has('vhost_parameters')) {
        $bodyArray = array_merge($bodyArray, get('vhost_parameters'));
    }

    $curl = curl_init();

    writeln('Sending deploy request to ' . $hostname);

    curl_setopt_array($curl, array(
        CURLOPT_URL => $vhostApiManagerHost . "/api/vhost/create",
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 30,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "POST",
        CURLOPT_POSTFIELDS => json_encode($bodyArray),
        CURLOPT_HTTPHEADER => array(
            "Accept: application/json",
            "Authorization: Bearer " . getVhostManagerApiAccessToken(),
            "Content-Type: application/json",
        ),
    ));

    $response = curl_exec($curl);
    $err = curl_error($curl);

    curl_close($curl);

    if ($err) {
        echo "cURL Error #:" . $err;
    } else {
        var_dump($response);
        $jsonResponse = json_decode($response, true);

        if (!empty($jsonResponse['success']) && $jsonResponse['success'] == true) {
            return true;
        } else {
            return false;
        }
    }
}

/**
 * Returns the API manager access token, uses one set in deploy.php if it is with the key vhostmanagerapi/access_token
 * otherwise uses default one
 * @return string
 */
function getVhostManagerApiAccessToken() {
    if (has('vhostmanagerapi/access_token')) {
        return get('vhostmanagerapi/access_token');
    }
    return 'eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsImp0aSI6IjQ0MjU5YTI0Yzg1ODg4MjAwODk1YmZlOGE4YzZhMjc2YmZhN2Y2YWZhMzc2Y2QwNWU0YjQ4YzQyYjRhOWI5YTJhMjU5NWI0MDA4ZDhiMDc1In0.eyJhdWQiOiIxIiwianRpIjoiNDQyNTlhMjRjODU4ODgyMDA4OTViZmU4YThjNmEyNzZiZmE3ZjZhZmEzNzZjZDA1ZTRiNDhjNDJiNGE5YjlhMmEyNTk1YjQwMDhkOGIwNzUiLCJpYXQiOjE1NjIyMjk5NzMsIm5iZiI6MTU2MjIyOTk3MywiZXhwIjoxNTkzODUyMzczLCJzdWIiOiI1Iiwic2NvcGVzIjpbXX0.FN0gAeF481u5TwQ8HWDbY9l6lwnBLflIz4go9Iqranq3heb8q5H7w_L-RcrV2tLQ7YmnC3cFOrGZpB1E21vh5wgn2IhBhboz5v2oZMhQ4KffDdjYrM9FmqbK7hvJrcp2knJLOw0nzCi64mU-DdjtQKuf-1rhRGo323NB2mIXpLh3Bh11c6YyE-XG8bPhKrDEnpclXSSN55bLj47FnyaGQ_Gwo1-gDYlDYBPn3GNFOJA70WlB506Xafyoe36eK6-sg19lbEA8DxtTylX9kcFqK9S0euE6tqkpP7fnLR9UnEu3zJuexD2Bg8XLsWzef6TeGtB3RzmhB6WWJVygHZzJ4LmPz1sGy8kqR_JeOBMflbLYN94kgxktROtZOuOjZPRUt9QG9G9iORecyqyeYiaIDuaZOef9IWLH56v9LIXR554VkSRsvAmj0zC2M6J-Ia7CSfSfsFjTn56kA2Sp1i8YcB-3o9VIhwLBz1OHPaJdt9jP61JupTojOrEMDCkXWnGok0MdTVYX8OhK4RuqVtR0AziUmaix76H8WOWEnFeNLAicbad4wU5t-1vxN43Ejqn_mC55Muj1Upg_xPNCx8u7q5n7ap5ikTqT21E6r7iQgsoAtItgyzPSAdTMIOGMP0KwLIFm-pPI4ry-OyTCqvZH-fEXGXv3K0oATS_LlFXzC7s';
}

/**
 * Gets the public directory path for a given application type
 *
 * @param $applicationType
 * @return string The correct relative path to the application public directory e.g. for Cakephp2 app/webroot, Laravel
 *         public
 */
function getApplicationPublicPath($applicationType) {
    switch ($applicationType) {
        case 'cake':
        case 'cakephp':
        case 'cakephp2':
            return 'app/webroot';
        case 'laravel':
        default:
            return 'public';
    }
}
